﻿using System;
using System.Linq;

using Microsoft.Xna.Framework.Input;

using Pong.Behaviours;

using Syntriax.Engine.Core;
using Syntriax.Engine.Core.Factory;
using Syntriax.Engine.Integration.MonoGame;
using Syntriax.Engine.Network;
using Syntriax.Engine.Physics2D;
using Syntriax.Engine.Systems.Tween;

namespace Pong;

public class GamePong : MonoGameWindow
{
    protected override void Initialize()
    {
        base.Initialize();

        if (Environment.GetCommandLineArgs().FirstOrDefault(x => x.CompareTo("-server") == 0) is not null)
        {
            LiteNetLibServer server = Universe.InstantiateUniverseObject().SetUniverseObject("Server").BehaviourController.AddBehaviour<LiteNetLibServer>();
            server.BehaviourController.AddBehaviour<NetworkManager>();
            server.Start(8888, 2);
            Window.Title = $"Server";
        }
        else
        {
            LiteNetLibClient client = Universe.InstantiateUniverseObject().SetUniverseObject("Client").BehaviourController.AddBehaviour<LiteNetLibClient>();
            client.BehaviourController.AddBehaviour<NetworkManager>();
            client.Connect("localhost", 8888);
            Window.Title = $"Client";

            DrawManager drawManager = Universe.InstantiateUniverseObject().SetUniverseObject("Draw Manager").BehaviourController.AddBehaviour<DrawManager>();
            Universe.InstantiateUniverseObject().SetUniverseObject("Shape Batcher", drawManager.UniverseObject).BehaviourController.AddBehaviour<ShapeBatcher>();
            Universe.InstantiateUniverseObject().SetUniverseObject("Sprite Batcher", drawManager.UniverseObject).BehaviourController.AddBehaviour<SpriteBatcher>();

            Universe.InstantiateUniverseObject().SetUniverseObject("Score Left")
                .BehaviourController.AddBehaviour<Transform2D>().SetTransform(position: new Vector2D(-250f, 250f), scale: Vector2D.One * .25f)
                .BehaviourController.AddBehaviour<TextScoreBehaviour>(true);

            Universe.InstantiateUniverseObject().SetUniverseObject("Score Right")
                .BehaviourController.AddBehaviour<Transform2D>().SetTransform(position: new Vector2D(250f, 250f), scale: Vector2D.One * .25f)
                .BehaviourController.AddBehaviour<TextScoreBehaviour>(false);
        }

        Universe.InstantiateUniverseObject().SetUniverseObject("Update Manager").BehaviourController.AddBehaviour<UpdateManager>();
        Universe.InstantiateUniverseObject().SetUniverseObject("Coroutine Manager").BehaviourController.AddBehaviour<CoroutineManager>();
        Universe.InstantiateUniverseObject().SetUniverseObject("Tween Manager").BehaviourController.AddBehaviour<TweenManager>();
        Universe.InstantiateUniverseObject().SetUniverseObject("Physics Engine 2D").BehaviourController.AddBehaviour<PhysicsEngine2D>();

        ////////////////////////////////////////////////////////////////////////////////////

        Universe.InstantiateUniverseObject().SetUniverseObject("Camera")
            .BehaviourController.AddBehaviour<Transform2D>()
            .BehaviourController.AddBehaviour<CameraController>()
            .BehaviourController.AddBehaviour<MonoGameCamera2DBehaviour>();

        ////////////////////////////////////////////////////////////////////////////////////

        PongManagerBehaviour pongManager = Universe.InstantiateUniverseObject().SetUniverseObject("Pong Game Manager")
            .BehaviourController.AddBehaviour<PongManagerBehaviour>(5);

        ////////////////////////////////////////////////////////////////////////////////////

        Universe.InstantiateUniverseObject().SetUniverseObject("Ball")
            .BehaviourController.AddBehaviour<Transform2D>().SetTransform(position: new Vector2D(0, 0f), scale: new Vector2D(10f, 10f))
            .BehaviourController.AddBehaviour<CircleBehaviour>(new Circle(Vector2D.Zero, 1f))
            .BehaviourController.AddBehaviour<BallBehaviour>()
            .BehaviourController.AddBehaviour<RigidBody2D>();

        ////////////////////////////////////////////////////////////////////////////////////

        IUniverseObject leftPaddle = UniverseObjectFactory.Instantiate().SetUniverseObject("Left Paddle");
        leftPaddle.BehaviourController.AddBehaviour<Transform2D>().SetTransform(position: new Vector2D(-468f, 0f), scale: new Vector2D(15f, 60f))
            .BehaviourController.AddBehaviour<ShapeBehaviour>(Shape2D.Square)
            .BehaviourController.AddBehaviour<RigidBody2D>().IsStatic = true;
        PaddleBehaviour leftPaddleBehaviour = BehaviourFactory.Instantiate<PaddleBehaviour>(Keys.W, Keys.S, 228f, -228f, 400f);
        leftPaddleBehaviour.Id = "lp";
        leftPaddle.BehaviourController.AddBehaviour(leftPaddleBehaviour);
        Universe.Register(leftPaddle);

        IUniverseObject rightPaddle = UniverseObjectFactory.Instantiate().SetUniverseObject("Right Paddle");
        rightPaddle.BehaviourController.AddBehaviour<Transform2D>().SetTransform(position: new Vector2D(468f, 0f), scale: new Vector2D(15f, 60f))
            .BehaviourController.AddBehaviour<ShapeBehaviour>(Shape2D.Square)
            .BehaviourController.AddBehaviour<RigidBody2D>().IsStatic = true;
        PaddleBehaviour rightPaddleBehaviour = BehaviourFactory.Instantiate<PaddleBehaviour>(Keys.Up, Keys.Down, 228f, -228f, 400f);
        rightPaddleBehaviour.Id = "rp";
        rightPaddle.BehaviourController.AddBehaviour(rightPaddleBehaviour);
        Universe.Register(rightPaddle);

        ////////////////////////////////////////////////////////////////////////////////////

        Universe.InstantiateUniverseObject().SetUniverseObject("Wall Top")
            .BehaviourController.AddBehaviour<Transform2D>().SetTransform(position: new Vector2D(0f, 308f), scale: new Vector2D(552f, 20f))
            .BehaviourController.AddBehaviour<ShapeBehaviour>(Shape2D.Square)
            .BehaviourController.AddBehaviour<RigidBody2D>().IsStatic = true;

        Universe.InstantiateUniverseObject().SetUniverseObject("Wall Bottom")
            .BehaviourController.AddBehaviour<Transform2D>().SetTransform(position: new Vector2D(0f, -308f), scale: new Vector2D(552f, 20f))
            .BehaviourController.AddBehaviour<ShapeBehaviour>(Shape2D.Square)
            .BehaviourController.AddBehaviour<RigidBody2D>().IsStatic = true;

        Universe.InstantiateUniverseObject().SetUniverseObject("Wall Right")
            .BehaviourController.AddBehaviour<Transform2D>().SetTransform(position: new Vector2D(532f, 0f), scale: new Vector2D(20f, 328f))
            .BehaviourController.AddBehaviour<WallScoreBehaviour>((Action)pongManager.ScoreToLeft)
            .BehaviourController.AddBehaviour<ShapeBehaviour>(Shape2D.Square)
            .BehaviourController.AddBehaviour<RigidBody2D>().IsStatic = true;

        Universe.InstantiateUniverseObject().SetUniverseObject("Wall Left")
            .BehaviourController.AddBehaviour<Transform2D>().SetTransform(position: new Vector2D(-532f, 0f), scale: new Vector2D(20f, 328f))
            .BehaviourController.AddBehaviour<WallScoreBehaviour>((Action)pongManager.ScoreToRight)
            .BehaviourController.AddBehaviour<ShapeBehaviour>(Shape2D.Square)
            .BehaviourController.AddBehaviour<RigidBody2D>().IsStatic = true;
    }
}
